@@ -18,7 +18,8 @@ from datashape import dshape, discover
 from .utils import ignoring
 from .expr import (Symbol, TableSymbol, symbol, ndim, shape)
 from .expr import (by, count, count_values, distinct, head, join, label, like,
-        mean, merge, nunique, relabel, selection, sort, summary, var, transform)
+        mean, merge, nunique, relabel, selection, sort, summary, var,
+        transform)
 from .expr import (date, datetime, day, hour, microsecond, millisecond, month,
         second, time, year)
 from .expr.arrays import (tensordot, transpose)
@@ -66,6 +67,9 @@ with ignoring(ImportError):
     from .pytables import *
     from .compute.pytables import *
 
+
+from .expr import concat  # Some module re-export toolz.concat and * catches it.
+
 restart_ordering()  # Restart multipledispatch ordering and do ordering
 
 inf = float('inf')
