@@ -45,28 +45,58 @@ http://docs.mongodb.org/manual/core/aggregation-pipeline/
 from __future__ import absolute_import, division, print_function
 
 import numbers
+import fnmatch
+
 from operator import attrgetter
 
 from pymongo.collection import Collection
 from pymongo.database import Database
 
-import fnmatch
 from datashape.predicates import isscalar
 from toolz import pluck, first, get, compose
 import toolz
 import datetime
 
-from ..expr import (Sort, count, nunique, nelements, Selection,
-                    mean, Reduction, Head, ReLabel, Distinct, ElemWise, By,
-                    Symbol, Projection, Field, sum, min, max, Gt, Lt, Ge, Le,
-                    Eq, Ne, And, Or, Summary, Like, Broadcast, DateTime,
-                    Microsecond, Date, Time, Expr, symbol, Arithmetic, floor,
-                    ceil, FloorDiv)
+from ..expr import (
+    And,
+    Arithmetic,
+    Broadcast,
+    By,
+    Distinct,
+    Eq,
+    Expr,
+    Field,
+    FloorDiv,
+    Ge,
+    Gt,
+    Head,
+    Le,
+    Lt,
+    Ne,
+    Or,
+    Projection,
+    Reduction,
+    SimpleSelection,
+    Sort,
+    Summary,
+    Like,
+    Symbol,
+    ceil,
+    count,
+    floor,
+    math,
+    max,
+    mean,
+    min,
+    nelements,
+    nunique,
+    sum,
+    symbol,
+)
 from ..expr.broadcast import broadcast_collect, Broadcastable
-from ..expr import math
 from ..expr.datetime import Day, Month, Year, Minute, Second, UTCFromTimestamp
+from ..expr.optimize import simple_selections
 from ..compatibility import _strtypes
-
 from ..dispatch import dispatch
 
 
@@ -121,7 +151,7 @@ class MongoQuery(object):
 
 @dispatch(Expr, (MongoQuery, Collection))
 def optimize(expr, seq):
-    return broadcast_collect(expr)
+    return broadcast_collect(simple_selections(expr), no_recurse=SimpleSelection)
 
 
 @dispatch(Head, MongoQuery)
@@ -207,7 +237,7 @@ def compute_up(t, q, **kwargs):
     return q.append({'$project': dict((col, 1) for col in t.fields)})
 
 
-@dispatch(Selection, MongoQuery)
+@dispatch(SimpleSelection, MongoQuery)
 def compute_up(expr, data, **kwargs):
     predicate = optimize(expr.predicate, data)
 
@@ -225,13 +255,6 @@ def compute_up(expr, data, **kwargs):
     return data.append({'$match': match(expr)})
 
 
-@dispatch(Like, MongoQuery)
-def compute_up(t, q, **kwargs):
-    pats = dict((name, {'$regex': fnmatch.translate(pattern)})
-                for name, pattern in t.patterns.items())
-    return q.append({'$match': pats})
-
-
 @dispatch(By, MongoQuery)
 def compute_up(t, q, **kwargs):
     if not isinstance(t.grouper, (Field, Projection, Symbol)):
@@ -435,10 +458,43 @@ def name(e):
         return e
 
 
-# Reflective binary operator, e.g. (x < y) -> (y > x)
-binop_swap = {Lt: Gt, Gt: Lt, Ge: Le, Le: Ge, Eq: Eq, Ne: Ne}
+opnames = {
+    Lt: '$lt',
+    Le: '$lte',
+    Gt: '$gt',
+    Ge: '$gte',
+    Ne: '$ne',
+    And: '$and',
+    Or: '$or',
+    Like: '$regex',
+}
 
 
+@dispatch((Lt, Le, Gt, Ge, Ne))
+def match(expr):
+    return {name(expr.lhs): {opnames[type(expr)]: expr.rhs}}
+
+
+@dispatch((And, Or))
+def match(expr):
+    return {opnames[type(expr)]: [match(expr.lhs), match(expr.rhs)]}
+
+
+@dispatch(Eq)
+def match(expr):
+    return {name(expr.lhs): name(expr.rhs)}
+
+
+@dispatch(Like)
+def match(expr):
+    return {
+        name(expr._child): {
+            opnames[type(expr)]: fnmatch.translate(expr.pattern)
+        }
+    }
+
+
+@dispatch(object)
 def match(expr):
     """ Match query for MongoDB
 
@@ -457,23 +513,6 @@ def match(expr):
     >>> match((x > 10) | (name == 'Alice'))
     {'$or': [{'x': {'$gt': 10}}, {'name': 'Alice'}]}
     """
-    if not isinstance(expr.lhs, Expr):
-        return match(binop_swap(type(expr))(expr.lhs, expr.rhs))
-    if isinstance(expr, Eq):
-        return {name(expr.lhs): name(expr.rhs)}
-    if isinstance(expr, Lt):
-        return {name(expr.lhs): {'$lt': expr.rhs}}
-    if isinstance(expr, Le):
-        return {name(expr.lhs): {'$lte': expr.rhs}}
-    if isinstance(expr, Gt):
-        return {name(expr.lhs): {'$gt': expr.rhs}}
-    if isinstance(expr, Ge):
-        return {name(expr.lhs): {'$gte': expr.rhs}}
-    if isinstance(expr, And):
-        return {'$and': [match(expr.lhs), match(expr.rhs)]}
-    if isinstance(expr, Or):
-        return {'$or': [match(expr.lhs), match(expr.rhs)]}
-    if isinstance(expr, Ne):
-        return {name(expr.lhs): {'$ne': expr.rhs}}
-    raise NotImplementedError("Matching not supported on expressions of type "
-                              "%r" % type(expr).__name__)
+    raise NotImplementedError(
+        'Matching not supported on expressions of type %r' % type(expr).__name__
+    )
