@@ -17,7 +17,6 @@ import fnmatch
 import operator
 import datetime
 import math
-import re
 
 from collections import Iterator
 from functools import partial
@@ -38,11 +37,10 @@ from ..expr import (Projection, Field, Broadcast, Map, Label, ReLabel,
                     By, Sort, Head, Apply, Summary, Like, IsIn,
                     DateTime, Date, Time, Millisecond, ElemWise,
                     Symbol, Slice, Expr, Arithmetic, ndim, DateTimeTruncate,
-                    UTCFromTimestamp, notnull)
+                    UTCFromTimestamp, notnull, UnaryMath, greatest, least)
 from ..expr import reductions
 from ..expr import count, nunique, mean, var, std
-from ..expr import (BinOp, UnaryOp, RealMath, IntegerMath, BooleanMath, USub,
-                    Not, nelements)
+from ..expr import BinOp, UnaryOp, USub, Not, nelements
 from ..compatibility import builtins, apply, unicode, _inttypes
 from .core import compute, compute_up, optimize, base
 
@@ -198,7 +196,7 @@ def rowfunc(expr):
     return partial(pydatetime.truncate, measure=expr.measure, unit=expr.unit)
 
 
-@dispatch((RealMath, IntegerMath, BooleanMath))
+@dispatch(UnaryMath)
 def rowfunc(expr):
     return getattr(math, type(expr).__name__)
 
@@ -222,6 +220,16 @@ def rowfunc(expr):
     return expr.op
 
 
+@dispatch(greatest)
+def rowfunc(_):
+    return max
+
+
+@dispatch(least)
+def rowfunc(_):
+    return min
+
+
 @dispatch(ElemWise, base)
 def compute_up(expr, data, **kwargs):
     return rowfunc(expr)(data)
@@ -315,6 +323,12 @@ def compute_up(t, seq, **kwargs):
     return filter(predicate, seq)
 
 
+@dispatch(Selection, Sequence, Sequence)
+def compute_up(expr, seq, predicate, **kwargs):
+    preds = iter(predicate)
+    return filter(lambda _: next(preds), seq)
+
+
 @dispatch(Reduction, Sequence)
 def compute_up(t, seq, **kwargs):
     if t.axis != (0,):
@@ -355,7 +369,7 @@ def compute_up(uop, x, **kwargs):
     return uop.op(x)
 
 
-@dispatch(RealMath, numbers.Real)
+@dispatch(UnaryMath, numbers.Real)
 def compute_up(f, n, **kwargs):
     return getattr(math, type(f).__name__)(n)
 
@@ -680,24 +694,11 @@ def compute_up(expr, data, **kwargs):
         return result
 
 
-def like_regex_predicate(expr):
-    regexes = dict((name, re.compile('^' + fnmatch.translate(pattern) + '$'))
-                   for name, pattern in expr.patterns.items())
-    regex_tup = [regexes.get(name, None) for name in expr.fields]
-
-    def predicate(tup):
-        for item, regex in zip(tup, regex_tup):
-            if regex and not regex.match(item):
-                return False
-        return True
-
-    return predicate
-
-
 @dispatch(Like, Sequence)
 def compute_up(expr, seq, **kwargs):
-    predicate = like_regex_predicate(expr)
-    return filter(predicate, seq)
+    def func(x, pattern=expr.pattern):
+        return fnmatch.fnmatch(x, pattern)
+    return map(func, seq)
 
 
 @dispatch(Slice, Sequence)
