@@ -11,11 +11,12 @@ from collections import Iterator, Iterable
 import blaze
 from blaze.compute.python import (nunique, mean, rrowfunc, rowfunc,
                                   reduce_by_funcs, optimize)
-from blaze import dshape
+from blaze import dshape, symbol, discover
 from blaze.compute.core import compute, compute_up, pre_compute
-from blaze.expr import (symbol, by, merge, join, count, distinct,
-                        Apply, sum, min, max, any, summary,
-                        count, std, head, transform)
+from blaze.expr import (
+    by, merge, join, distinct, sum, min, max, any, summary, count, std, head,
+    transform, greatest, least
+)
 import numpy as np
 
 from blaze import cos, sin
@@ -627,14 +628,18 @@ def test_scalar_arithmetic():
 
 def test_like():
     t = symbol('t', 'var * {name: string, city: string}')
-    data = [('Alice Smith', 'New York'),
-            ('Bob Smith', 'Chicago'),
-            ('Alice Walker', 'LA')]
+    data = [
+        ('Alice Smith', 'New York'),
+        ('Bob Smith', 'Chicago'),
+        ('Alice Walker', 'LA')
+    ]
 
-    assert list(compute(t.like(name='Alice*'), data)) == [data[0], data[2]]
-    assert list(compute(t.like(name='lice*'), data)) == []
-    assert list(compute(t.like(name='*Smith*'), data)) == [data[0], data[1]]
-    assert list(compute(t.like(name='*Smith*', city='New York'), data)) == [data[0]]
+    assert list(compute(t[t.name.like('Alice*')], data)) == [data[0], data[2]]
+    assert list(compute(t[t.name.like('lice*')], data)) == []
+    assert list(compute(t[t.name.like('*Smith*')], data)) == [data[0], data[1]]
+    assert list(
+        compute(t[t.name.like('*Smith*') & t.city.like('New York')], data)
+    ) == [data[0]]
 
 
 def test_datetime_comparison():
@@ -867,3 +872,19 @@ def test_isin(keys):
     result = list(compute(expr, data))
     expected = [el for el in data if el[0] in keys]
     assert result == expected
+
+
+def test_greatest():
+    s_data, t_data = [1, 2], [2, 1]
+    s, t = symbol('s', discover(s_data)), symbol('t', discover(t_data))
+    result = compute(greatest(s, t), {s: s_data, t: t_data})
+    expected = np.maximum(s_data, t_data).tolist()
+    assert list(result) == expected
+
+
+def test_least():
+    s_data, t_data = [1, 2], [2, 1]
+    s, t = symbol('s', discover(s_data)), symbol('t', discover(t_data))
+    result = compute(least(s, t), {s: s_data, t: t_data})
+    expected = np.minimum(s_data, t_data).tolist()
+    assert list(result) == expected
