@@ -1,5 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
+import numbers
+import numpy as np
 from functools import partial
 from itertools import chain
 
@@ -10,7 +12,7 @@ from datashape import (
 from datashape.predicates import isscalar, iscollection, isrecord
 from toolz import (
     isdistinct, frequencies, concat as tconcat, unique, get, first, compose,
-    keymap,
+    keymap
 )
 import toolz.curried.operator as op
 from odo.utils import copydoc
@@ -24,7 +26,7 @@ from ..utils import listpack
 
 __all__ = ['Sort', 'Distinct', 'Head', 'Merge', 'IsIn', 'isin', 'distinct',
            'merge', 'head', 'sort', 'Join', 'join', 'transform', 'Concat',
-           'concat', 'Tail', 'tail']
+           'concat', 'Tail', 'tail', 'Shift', 'shift']
 
 
 class Sort(Expr):
@@ -44,8 +46,7 @@ class Sort(Expr):
     """
     __slots__ = '_hash', '_child', '_key', 'ascending'
 
-    @property
-    def dshape(self):
+    def _dshape(self):
         return self._child.dshape
 
     @property
@@ -132,8 +133,7 @@ class Distinct(Expr):
     """
     __slots__ = '_hash', '_child', 'on'
 
-    @property
-    def dshape(self):
+    def _dshape(self):
         return datashape.var * self._child.dshape.measure
 
     @property
@@ -174,8 +174,7 @@ def distinct(expr, *on):
 class _HeadOrTail(Expr):
     __slots__ = '_hash', '_child', 'n'
 
-    @property
-    def dshape(self):
+    def _dshape(self):
         return self.n * self._child.dshape.subshape[0]
 
     def _len(self):
@@ -256,21 +255,17 @@ def schema_concat(exprs):
 
     In the case of Units, the name is taken from expr.name
     """
-    names, values = [], []
+    new_fields = []
     for c in exprs:
         schema = c.schema[0]
-        if isinstance(schema, Option):
-            schema = schema.ty
         if isinstance(schema, Record):
-            names.extend(schema.names)
-            values.extend(schema.types)
-        elif isinstance(schema, Unit):
-            names.append(c._name)
-            values.append(schema)
+            new_fields.extend(schema.fields)
+        elif isinstance(schema, (Unit, Option)):
+            new_fields.append((c._name, schema))
         else:
             raise TypeError("All schemas must have Record or Unit shape."
-                            "\nGot %s" % c.schema[0])
-    return dshape(Record(list(zip(names, values))))
+                            "\nGot %s" % schema)
+    return dshape(Record(new_fields))
 
 
 class Merge(ElemWise):
@@ -286,8 +281,7 @@ class Merge(ElemWise):
     """
     __slots__ = '_hash', '_child', 'children'
 
-    @property
-    def schema(self):
+    def _schema(self):
         return schema_concat(self.children)
 
     @property
@@ -327,11 +321,7 @@ def merge(*exprs, **kwargs):
             return v.label(k)
     # Get common sub expression
     exprs += tuple(label(v, k) for k, v in sorted(kwargs.items(), key=first))
-    try:
-        child = common_subexpression(*exprs)
-    except Exception:
-        raise ValueError("No common subexpression found for input expressions")
-
+    child = common_subexpression(*exprs)
     result = Merge(child, exprs)
 
     if not isdistinct(result.fields):
@@ -399,7 +389,7 @@ class Join(Expr):
     blaze.expr.collections.Merge
     """
     __slots__ = (
-        '_hash', 'lhs', 'rhs', '_on_left', '_on_right', 'how', 'suffixes',
+        '_hash', 'lhs', 'rhs', '_on_left', '_on_right', 'how', 'suffixes'
     )
     __inputs__ = 'lhs', 'rhs'
 
@@ -417,8 +407,7 @@ class Join(Expr):
             return list(on_right)
         return on_right
 
-    @property
-    def schema(self):
+    def _schema(self):
         """
 
         Examples
@@ -440,6 +429,7 @@ class Join(Expr):
         >>> join(a, b, 'x').fields
         ['x', 'y_left', 'y_right']
         """
+
         option = lambda dt: dt if isinstance(dt, Option) else Option(dt)
 
         on_left = self.on_left
@@ -494,8 +484,7 @@ class Join(Expr):
 
         return dshape(Record(chain(joined, left, right)))
 
-    @property
-    def dshape(self):
+    def _dshape(self):
         # TODO: think if this can be generalized
         return var * self.schema
 
@@ -620,8 +609,7 @@ class Concat(Expr):
     __slots__ = '_hash', 'lhs', 'rhs', 'axis'
     __inputs__ = 'lhs', 'rhs'
 
-    @property
-    def dshape(self):
+    def _dshape(self):
         axis = self.axis
         ldshape = self.lhs.dshape
         lshape = ldshape.shape
@@ -694,8 +682,7 @@ class IsIn(ElemWise):
     """
     __slots__ = '_hash', '_child', '_keys'
 
-    @property
-    def schema(self):
+    def _schema(self):
         return datashape.bool_
 
     def __str__(self):
@@ -712,8 +699,49 @@ def isin(expr, keys):
     return IsIn(expr, frozenset(keys))
 
 
+class Shift(Expr):
+    """ Shift a column backward or forward by N elements
+
+    Parameters
+    ----------
+    expr : Expr
+        The expression to shift. This expression's dshape should be columnar
+    n : int
+        The number of elements to shift by. If n < 0 then shift backward,
+        if n == 0 do nothing, else shift forward.
+    """
+    __slots__ = '_hash', '_child', 'n'
+
+    def _schema(self):
+        measure = self._child.schema.measure
+
+        # if we are not shifting or we are already an Option type then return
+        # the child's schema
+        if not self.n or isinstance(measure, Option):
+            return measure
+        else:
+            return Option(measure)
+
+    def _dshape(self):
+        return DataShape(*(self._child.dshape.shape + tuple(self.schema)))
+
+    def __str__(self):
+        return '%s(%s, n=%d)' % (
+            type(self).__name__.lower(), self._child, self.n
+        )
+
+
+@copydoc(Shift)
+def shift(expr, n):
+    if not isinstance(n, (numbers.Integral, np.integer)):
+        raise TypeError('n must be an integer')
+    return Shift(expr, n)
+
+
 dshape_method_list.extend([
     (iscollection, set([sort, head, tail])),
-    (lambda ds: len(ds.shape) == 1, set([distinct])),
+    (lambda ds: len(ds.shape) == 1, set([distinct, shift])),
+    (lambda ds: (len(ds.shape) == 1 and
+                 isscalar(getattr(ds.measure, 'key', ds.measure))), set([isin])),
     (lambda ds: len(ds.shape) == 1 and isscalar(ds.measure), set([isin])),
 ])
