@@ -1,8 +1,10 @@
 import pytest
 
 from datashape import dshape
+from datashape.util.testing import assert_dshape_equal
 
 from blaze.expr import symbol
+from blaze.expr.core import common_subexpression
 from blaze.expr.collections import merge, join, transform, concat
 from blaze.utils import raises
 from blaze.compatibility import builtins
@@ -20,10 +22,25 @@ def test_merge():
     assert expr.y.isidentical(e.x.label('y'))
 
 
+def test_merge_options():
+    s = symbol('s', 'var * {a: ?A, b: ?B}')
+
+    merged = merge(a=s.a, b=s.b)
+    assert_dshape_equal(merged.dshape, dshape('var * {a: ?A, b: ?B}'))
+    assert_dshape_equal(merged.a.dshape, dshape('var * ?A'))
+    assert_dshape_equal(merged.b.dshape, dshape('var * ?B'))
+
+
 def test_merge_on_single_argument_is_noop():
     assert merge(t.name).isidentical(t.name)
 
 
+def test_merge_common_subexpression():
+    t = symbol('t', 'var * {a: float64}')
+    result = common_subexpression(t.a - t.a % 3, t.a % 3)
+    assert result.isidentical(t.a)
+
+
 def test_transform():
     t = symbol('t', '5 * {x: int, y: int}')
     expr = transform(t, z=t.x + t.y)
@@ -187,3 +204,13 @@ def test_concat_axis_too_great():
 
     with pytest.raises(ValueError):
         concat(a, b, axis=2)
+
+
+def test_shift():
+    t = symbol('t', 'var * float64')
+    assert t.shift(1).dshape == dshape('var * ?float64')
+    assert t.shift(0).dshape == t.dshape
+    assert t.shift(-1).dshape == dshape('var * ?float64')
+    assert repr(t.shift(1)) == 'shift(t, n=1)'
+    assert repr(t.shift(0)) == 'shift(t, n=0)'
+    assert repr(t.shift(-1)) == 'shift(t, n=-1)'
