@@ -1,7 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
 from .server import Server, to_tree, from_tree, api
-from .spider import spider, from_yaml
+from .spider import data_spider, from_yaml
 from .client import Client
 from .serialization import (
     SerializationFormat,
@@ -16,7 +16,7 @@ __all__ = [
     'Client',
     'SerializationFormat',
     'Server',
-    'spider',
+    'data_spider',
     'from_yaml',
     'all_formats',
     'api',
