@@ -1,30 +1,26 @@
 from __future__ import absolute_import, division, print_function
 
-import os
 import datetime
+from collections import Iterator
+from itertools import islice
+import os
 import re
-from weakref import WeakKeyDictionary
 
 try:
-    from cytoolz import nth, memoize, unique, concat, first, drop
+    from cytoolz import nth, unique, concat, first, drop
 except ImportError:
-    from toolz import nth, memoize, unique, concat, first, drop
-
-from toolz.curried.operator import setitem
-
-from itertools import islice
-from collections import Iterator
+    from toolz import nth, unique, concat, first, drop
 
+import numpy as np
 # these are used throughout blaze, don't remove them
 from odo.utils import tmpfile, filetext, filetexts, raises, keywords, ignoring
-
 import pandas as pd
 import psutil
-import numpy as np
+import sqlalchemy as sa
+from toolz.curried.operator import setitem
 
 # Imports that replace older utils.
 from .compatibility import map, zip
-
 from .dispatch import dispatch
 
 
@@ -233,26 +229,42 @@ def _read_timedelta(ds):
 
 
 def normalize(s):
-    s = ' '.join(s.strip().split()).lower()
-    s = re.sub(r'(alias)_?\d*', r'\1', s)
-    return re.sub(r'__([A-Za-z_][A-Za-z_0-9]*)', r'\1', s)
-
-
-def weakmemoize(f):
-    """Memoize ``f`` with a ``WeakKeyDictionary`` to allow the arguments
-    to be garbage collected.
+    """Normalize a sql expression for comparison in tests.
 
     Parameters
     ----------
-    f : callable
-        The function to memoize.
+    s : str or Selectable
+        The expression to normalize. If this is a selectable, it will be
+        compiled with literals inlined.
 
     Returns
     -------
-    g : callable
-        ``f`` with weak memoiza
+    cs : Any
+        An object that can be compared against another normalized sql
+        expression.
     """
-    return memoize(f, cache=WeakKeyDictionary())
+    if isinstance(s, sa.sql.Selectable):
+        s = literal_compile(s)
+    s = re.sub(r'(\(|\))', r' \1 ', s)       # normalize spaces around parens
+    s = ' '.join(s.strip().split()).lower()  # normalize whitespace and case
+    s = re.sub(r'(alias)_?\d*', r'\1', s)    # normalize aliases
+    return re.sub(r'__([A-Za-z_][A-Za-z_0-9]*)', r'\1', s)
+
+
+def literal_compile(s):
+    """Compile a sql expression with bind params inlined as literals.
+
+    Parameters
+    ----------
+    s : Selectable
+        The expression to compile.
+
+    Returns
+    -------
+    cs : str
+        An equivalent sql string.
+    """
+    return str(s.compile(compile_kwargs={'literal_binds': True}))
 
 
 def ordered_intersect(*sets):
@@ -279,3 +291,22 @@ def ordered_intersect(*sets):
     """
     common = frozenset.intersection(*map(frozenset, sets))
     return (x for x in unique(concat(sets)) if x in common)
+
+
+class attribute(object):
+    """An attribute that can be overridden by instances.
+    This is like a non data descriptor property.
+
+    Parameters
+    ----------
+    f : callable
+        The function to execute.
+    """
+    def __init__(self, f):
+        self._f = f
+
+    def __get__(self, instance, owner):
+        if instance is None:
+            return self
+
+        return self._f(instance)
